/*
	Milyn - Copyright (C) 2006

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License (version 2.1) as published by the Free Software
	Foundation.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

	See the GNU Lesser General Public License for more details:
	http://www.gnu.org/licenses/lgpl.txt
*/
package example.beans;

import org.milyn.javabean.DataDecoder;
import org.milyn.javabean.DataDecodeException;
import org.milyn.cdr.SmooksResourceConfiguration;
import org.milyn.cdr.SmooksConfigurationException;

import java.util.regex.Pattern;
import java.util.Vector;
import java.util.List;

/**
 * Decoder for the Tracking number history.
 * @author <a href="mailto:tom.fennelly@gmail.com">tom.fennelly@gmail.com</a>
 */
public class TrackingNumberHistoryDecoder implements DataDecoder {
    
    private static Pattern lineSplitter = Pattern.compile("$", Pattern.MULTILINE);

    public void setConfiguration(SmooksResourceConfiguration resourceConfig) throws SmooksConfigurationException {
    }

    public Object decode(String historyText) throws DataDecodeException {
        // break the history up line by line - 1 tracking-number per line
        String[] unparsedTrackingNumber = lineSplitter.split(historyText);
        List<TrackingNumber> tnList = new Vector<TrackingNumber>(unparsedTrackingNumber.length);
        TrackingNumber[] trackingNumbers;

        // iterate over and parse the tracking-number lines
        for (int i = 0; i < unparsedTrackingNumber.length; i++) {
            String[] tokens = unparsedTrackingNumber[i].trim().split(":");

            if(tokens.length == 2) {
                TrackingNumber trackingNumber = new TrackingNumber();

                trackingNumber.setShipperID(tokens[0]);
                trackingNumber.setShipmentNumber(tokens[1]);
                tnList.add(trackingNumber);
            }
        }

        trackingNumbers = new TrackingNumber[tnList.size()];
        tnList.toArray(trackingNumbers);

        return trackingNumbers;
    }
}
